﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;

namespace Torture
{
    public class BearTrapBehaviour : MonoBehaviour
    {
        public Sprite[] sprites = new Sprite[3]; // 0 left 1 plate 2 right
        public GameObject pressurePlate;
        public GameObject leftBlade;
        public GameObject rightBlade;

        public HingeJoint2D jointLeftBlade;
        public HingeJoint2D jointRightBlade;
        public bool hooked = false;

        public GameObject target;

        public bool targetFixed = false;
        private void Start()
        {
            gameObject.GetComponent<Rigidbody2D>().freezeRotation = true;
            Utility.SetSprite(gameObject, sprites[1]);
            gameObject.FixColliders();
            if(rightBlade == null)
            {
                InitBlades();
            }
            gameObject.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("reloadTrap", "Charge the trap", "Charge the trap", () => StartCoroutine(UnhookedAfterSeconds(0))));
            gameObject.GetComponent<PhysicalBehaviour>().RefreshOutline();
            leftBlade.transform.localScale = new Vector3(1, 1, 1);
            rightBlade.transform.localScale = new Vector3(1, 1, 1);
            leftBlade.GetPhysicalBehaviour().Selectable = false;
            rightBlade.GetPhysicalBehaviour().Selectable = false;
        }
        private void InitBlades()
        {
            pressurePlate = gameObject;
            leftBlade = ModAPI.CreatePhysicalObject("leftBlade", null);
            leftBlade.transform.SetParent(pressurePlate.transform);
            rightBlade = ModAPI.CreatePhysicalObject("rightBlade", null);
            rightBlade.transform.SetParent(pressurePlate.transform);

            Rigidbody2D plateRigidBody = pressurePlate.GetComponent<Rigidbody2D>();

            Utility.SetSprite(leftBlade, sprites[0]);
            Utility.SetSprite(rightBlade, sprites[2]);

            jointLeftBlade = leftBlade.AddComponent<HingeJoint2D>();
            jointLeftBlade.connectedBody = plateRigidBody;
            jointRightBlade = rightBlade.AddComponent<HingeJoint2D>();
            jointRightBlade.connectedBody = plateRigidBody;

            Utility.SetPositionWithOffSet(rightBlade.transform, pressurePlate.transform, 0.1968f);
            Utility.SetPositionWithOffSet(leftBlade.transform, pressurePlate.transform, -0.1968f);

            jointLeftBlade.useLimits = true;
            jointRightBlade.useLimits = true;

            jointLeftBlade.limits = new JointAngleLimits2D() 
            { 
                min = 0, 
                max = 75 
            };
            jointRightBlade.limits = new JointAngleLimits2D() 
            { 
                min = 0,
                max = -75
            };

            jointLeftBlade.autoConfigureConnectedAnchor = false;
            jointRightBlade.autoConfigureConnectedAnchor = false;
            
            jointLeftBlade.anchor = new Vector2(0.1968f, 0);
            jointRightBlade.anchor = new Vector2(-0.1968f, 0);

            jointLeftBlade.connectedAnchor = new Vector2(0, 0);
            jointRightBlade.connectedAnchor = new Vector2(0, 0);

            jointLeftBlade.useMotor = true;
            jointRightBlade.useMotor = true;

            Physics2D.IgnoreCollision(leftBlade.GetComponent<Collider2D>(), rightBlade.GetComponent<Collider2D>(), true);

            var trapPlateBeh = gameObject.AddComponent<BearTrapPlate>();
            trapPlateBeh.trapBehaviour = this;
            var bladeLeftBeh = leftBlade.AddComponent<BearTrapBlade>();
            bladeLeftBeh.pointOffset = new Vector2(-0.0712f, 0.0515f);
            bladeLeftBeh.trapBehaviour = this;
            var bladeRightBeh = rightBlade.AddComponent<BearTrapBlade>();
            bladeRightBeh.trapBehaviour = this;
            bladeRightBeh.pointOffset = new Vector2(0.0712f, 0.0515f);
        }
        public void Hooked(PhysicalBehaviour physicalBehaviour)
        {
            //gameObject.transform.rotation = physicalBehaviour.gameObject.transform.rotation;
            target = physicalBehaviour.gameObject;
            hooked = true;
            if (physicalBehaviour.TryGetComponent<LimbBehaviour>(out LimbBehaviour limb) && UnityEngine.Random.Range(0, 11) > 7)
            {
                limb.Slice();
            }
            StartCoroutine(UnhookedAfterSeconds(3));
            
        }
        private IEnumerator UnhookedAfterSeconds(float seconds)
        {
            yield return new WaitForSeconds(seconds);
            if(target == null || !targetFixed)
            {
                if(target != null)
                {
                    Physics2D.IgnoreCollision(gameObject.GetComponent<Collider2D>(), target.GetComponent<Collider2D>(), false);
                }
                target = null;
                hooked = false;

                leftBlade.GetComponent<BearTrapBlade>().Unhooked();
                rightBlade.GetComponent<BearTrapBlade>().Unhooked();
            }
        }
        public void Update()
        {
            targetFixed = leftBlade.GetComponent<BearTrapBlade>().targetJoint != null && rightBlade.GetComponent<BearTrapBlade>().targetJoint != null ? true : false;
        }
    }
    public class BearTrapPlate : MonoBehaviour
    {
        public BearTrapBehaviour trapBehaviour;
        public CircleCollider2D circleCollider;
        private void Start()
        {
            circleCollider = gameObject.AddComponent<CircleCollider2D>();
            circleCollider.isTrigger = true;
            circleCollider.offset = new Vector2(0, 0.1f);
        }
        public void OnCollisionEnter2D(Collision2D collision)
        {
            if(collision.gameObject.TryGetComponent<PhysicalBehaviour>(out PhysicalBehaviour physicalBehaviour) && !trapBehaviour.hooked && Utility.InCollider(circleCollider).Contains(collision.collider))
            {
                if(LayerMask.LayerToName(physicalBehaviour.gameObject.layer) != "Bounds")
                {
                    trapBehaviour.Hooked(physicalBehaviour);
                    Physics2D.IgnoreCollision(gameObject.GetComponent<Collider2D>(), trapBehaviour.target.GetComponent<Collider2D>());
                }
            }
        }
    }
    public class BearTrapBlade : MonoBehaviour
    {
        public Vector2 pointOffset;
        public GameObject pointContact;
        public HingeJoint2D targetJoint;
        public BearTrapBehaviour trapBehaviour;
        public HingeJoint2D hingeJoint;
        public Collider2D[] inCircle;
        public float speedHooked = 3000f;

        private void Start()
        {
            hingeJoint = gameObject.GetComponent<HingeJoint2D>();
            pointContact = new GameObject("pointContact");
            pointContact.transform.SetParent(gameObject.transform);
            var position = gameObject.transform.position;
            pointContact.transform.position = new Vector2(pointOffset.x + position.x, pointOffset.y + position.y);
        }
        public void Unhooked()
        {
            UnityEngine.Object.Destroy(targetJoint);
            ResetMotor();
        }
        public void Update()
        {
            if (trapBehaviour.hooked && !trapBehaviour.targetFixed && targetJoint == null)
            {
                var speedMotor = hingeJoint.anchor.x > 0 ? speedHooked : -speedHooked;
                hingeJoint.motor = Utility.SetMotorForce(hingeJoint.motor, speedMotor);
                inCircle = Physics2D.OverlapCircleAll(pointContact.transform.position, 0.03f);
                if (inCircle.Contains(trapBehaviour.target.gameObject.GetComponent<Collider2D>()))
                {
                    if(trapBehaviour.TryGetComponent<LimbBehaviour>(out LimbBehaviour limb))
                    {
                        limb.SkinMaterialHandler.AddDamagePoint(DamageType.Stab, pointContact.transform.position, 15f);
                    }
                    targetJoint = gameObject.AddComponent<HingeJoint2D>();
                    targetJoint.connectedBody = trapBehaviour.target.GetComponent<Rigidbody2D>();
                    targetJoint.enableCollision = false;
                }
            }
            if (trapBehaviour.targetFixed)
            {
                StopMotor();
            }
            if(Math.Abs(hingeJoint.jointAngle) > 89 && targetJoint == null && trapBehaviour.target != null)
            {
                //StopMotor();
            }
        }
        public void OnCollisionEnter2D(Collision2D collision)
        {
            if (inCircle.Contains(collision.collider) || trapBehaviour.target == collision.gameObject)
            {
                if(collision.gameObject.TryGetComponent<LimbBehaviour>(out LimbBehaviour limb))
                {
                    limb.Shot(new Shot()
                    {
                        point = collision.contacts[0].point,
                        damage = 20f
                    });
                    limb.CirculationBehaviour.Cut(collision.contacts[0].point, collision.contacts[0].point + hingeJoint.anchor);
                    limb.SkinMaterialHandler.AddDamagePoint(DamageType.Stab, collision.contacts[0].point, UnityEngine.Random.Range(5f, 15f));
                }
            }
        }
        public void StopMotor()
        {
            hingeJoint.motor = Utility.SetMotorForce(hingeJoint.motor, 0f, 0f);
            hingeJoint.useMotor = false;
            //hingeJoint.useLimits = false;
        }
        public void ResetMotor()
        {
            hingeJoint.motor = Utility.SetMotorForce(hingeJoint.motor, 0f);
            hingeJoint.useMotor = true;
        }
    }
}
