﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;

namespace Torture
{
    public class CircularTable : MonoBehaviour
    {
        public Sprite[] sprites = new Sprite[] { }; // 0 - table 1 - disk
        public AudioClip[] audioClips = new AudioClip[] { };
        public SliderJoint2D sliderJoint;
        public DamagableMachineryBehaviour damageBehaviour;
        public CircularTableDisk diskBehaviour;
        public AudioSource audioSource;
        private void Start()
        {
            gameObject.transform.localScale = gameObject.transform.localScale.GetModuleVector();
            gameObject.SetSprite(sprites[0], true, "Foreground");
            var diskObjectHolder = Utility.CreateChildObject("diskHolder", gameObject.transform);
            diskObjectHolder.SetSprite(sprites[1]);
            diskObjectHolder.InitializePhysicalComponent();
            var diskPhysicBehaviour = diskObjectHolder.GetComponent<PhysicalBehaviour>();
            var diskLocalProperties = diskPhysicBehaviour.Properties.ShallowClone();
            diskLocalProperties.Sharp = true;
            diskLocalProperties.SharpAxes = new SharpAxis[]
            {
                new SharpAxis(default, 0, 100, true, true)
            };
            diskPhysicBehaviour.Properties = diskLocalProperties;
            Utility.SetPositionWithOffSet(diskObjectHolder.transform, gameObject.transform);

            sliderJoint = gameObject.AddComponent<SliderJoint2D>();
            sliderJoint.autoConfigureAngle = false;
            sliderJoint.autoConfigureConnectedAnchor = false;
            sliderJoint.connectedBody = diskObjectHolder.GetComponent<Rigidbody2D>();
            sliderJoint.angle = 0;
            sliderJoint.limits = new JointTranslationLimits2D()
            {
                min = -0.8f,
                max = 0.8f
            };
            sliderJoint.useLimits = true;

            UnityEngine.Object.Destroy(diskObjectHolder.GetComponent<SpriteRenderer>());
            var diskObject = Utility.CreateChildObject("disk", diskObjectHolder.transform);
            diskObject.SetSprite(sprites[1]);

            diskBehaviour = diskObjectHolder.AddComponent<CircularTableDisk>();

            Utility.SetPositionWithOffSet(diskObject.transform, diskObjectHolder.transform);
            var damagableMachineryExample = ModAPI.FindSpawnable("Industrial Piston").Prefab.GetComponent<DamagableMachineryBehaviour>().ShallowClone();
            damageBehaviour = gameObject.AddComponent<DamagableMachineryBehaviour>();
            damageBehaviour.BreakDamage = 10;
            damageBehaviour.Waterproof = false;
            damageBehaviour.FirearmDamageMultiplier = 1.1f;
            damageBehaviour.ExplosionDamageMultiplier = 1.3f;
            damageBehaviour.BehavioursToToggle = new MonoBehaviour[] { };
            damageBehaviour.GameObjectsToToggle = new GameObject[] { };
            damageBehaviour.ExplosionPrefab = damagableMachineryExample.ExplosionPrefab;
            

            var actOnCollide = gameObject.AddComponent<ActOnCollideOwn>();
            actOnCollide.DispatchChance = 1f;
            actOnCollide.ImpactForceThreshold = 70f;
            actOnCollide.debug = false;
            actOnCollide.Actions = (collision) =>
            {
                if (!damageBehaviour.Destroyed)
                {
                    damageBehaviour.Slice();
                    diskBehaviour.activate = false;
                }
            };
            diskBehaviour.damagableMachinery = damageBehaviour;
            diskBehaviour.diskObject = diskObject;
            diskBehaviour.collider = diskObjectHolder.GetComponent<Collider2D>();
            var diskActOnCollide = diskObjectHolder.AddComponent<ActOnCollideOwn>();
            diskActOnCollide.DispatchChance = 1f;
            diskActOnCollide.ImpactForceThreshold = 0f;
            diskActOnCollide.Actions = (collision) =>
            {
                if (diskBehaviour.speed > 300)
                {
                    collision.rigidbody.AddForce(Vector2.right, ForceMode2D.Impulse);
                    if (collision.gameObject.TryGetComponent<LimbBehaviour>(out LimbBehaviour limb))
                    {
                        limb.Slice();
                    }
                    if(collision.gameObject.TryGetComponent<DestroyableBehaviour>(out DestroyableBehaviour destroyableBehaviour))
                    {
                        destroyableBehaviour.Break(Vector2.right);
                    }
                    if(collision.gameObject.TryGetComponent<DamagableMachineryBehaviour>(out DamagableMachineryBehaviour damagableMachineryBehaviour))
                    {
                        damagableMachineryBehaviour.Slice();
                    }
                }
            };
            var localScaleX = gameObject.transform.localScale.x;
            sliderJoint.motor = Utility.SetMotorForce(sliderJoint.motor, 1 * localScaleX);
            gameObject.GetComponent<Rigidbody2D>().mass = 10f;
            gameObject.UpdateOutline();
            audioSource = diskObjectHolder.AddComponent<AudioSource>();
            audioSource.clip = audioClips[0];
            audioSource.loop = true;
            diskObjectHolder.AddComponent<AudioSourceTimeScaleBehaviour>();
            diskBehaviour.audioSource = audioSource;
        }
        private void Update()
        {
            if (diskBehaviour.activate && !damageBehaviour.Destroyed)
            {
                sliderJoint.useMotor = true;
            }
            else
            {
                sliderJoint.useMotor = false;
            }
            var localScaleX = gameObject.transform.localScale.x;
            sliderJoint.limits = new JointTranslationLimits2D()
            {
                min = -0.8f * localScaleX,
                max = 0.8f * localScaleX
            };
            if(!damageBehaviour.Destroyed && diskBehaviour.activate)
            {
                if(sliderJoint.jointTranslation >= sliderJoint.limits.max)
                {
                    sliderJoint.motor = Utility.SetMotorForce(sliderJoint.motor, -1 * localScaleX);
                }
                else if(sliderJoint.jointTranslation <= sliderJoint.limits.min)
                {
                    sliderJoint.motor = Utility.SetMotorForce(sliderJoint.motor, 1 * localScaleX);
                }
            }
        }
    }
    public class CircularTableDisk : MonoBehaviour
    {
        public bool activate;
        public DamagableMachineryBehaviour damagableMachinery;
        public float speed = 5f;
        public float maxSpeed = 1000f;
        public float minSpeed = 0f;
        public GameObject diskObject;
        public Collider2D collider;
        public AudioSource audioSource;
        private void Use()
        {
            if (!damagableMachinery.Destroyed)
            {
                activate = !activate;
                if (activate)
                {
                    //audioSource.Play();
                }
                else
                {
                    //audioSource.Stop();
                }
            }
        }
        private void Update()
        {
            if (!damagableMachinery.Destroyed && activate && speed < maxSpeed)
            {
                speed = speed + 100 * Time.deltaTime;
            }
            else if(speed > minSpeed)
            {
                speed = speed - 100 * Time.deltaTime;
                if(speed < minSpeed)
                {
                    speed = 0;
                }
            }
            var localEuler = diskObject.transform.localEulerAngles;
            diskObject.transform.localEulerAngles = new Vector3(localEuler.x, localEuler.y, localEuler.z + speed * Time.deltaTime);

            if(speed > 310)
            {
                collider.isTrigger = false;
            }
            if(speed < 310)
            {
                collider.isTrigger = true;
            }
        }
    }
}
