﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;

namespace Torture
{
    public class Freezer : MonoBehaviour
    {
        public Sprite[] sprites = new Sprite[] { }; // 0 - sides 1 - tf 2 - f 3 - door
        public bool activate = false;
        public bool foregroundTransparent = false;
        public DamagableMachineryBehaviour damageBehaviour;
        public BoxCollider2D internalCollider;
        public float maxTemperature = -25f;
        public float speed = 0.3f;
        public SpriteRenderer foregroundSpriteRenderer;
        public Material defaultMaterial;
        public Material brokenMaterial;
        private void Start()
        {
            //gameObject.transform.localScale = Utility.GetModuleVector(gameObject.transform.localScale);
            float colliderScale = 2.85f;
            gameObject.SetSprite(sprites[0], false);
            UnityEngine.Object.Destroy(gameObject.GetComponent<Collider2D>());
            var colliderSides = gameObject.AddComponent<PolygonCollider2D>();
            colliderSides.points = new Vector2[]
            {
                new Vector2(0.3801441f, -0.184207f) * colliderScale,
                new Vector2(-0.3827801f, -0.1840997f) * colliderScale,
                new Vector2(-0.3824062f, 0.1963587f) * colliderScale,
                new Vector2(-0.4081161f, 0.1964672f) * colliderScale,
                new Vector2(-0.4080208f, -0.2066847f) * colliderScale,
                new Vector2(0.4058254f, -0.2070037f) * colliderScale,
                new Vector2(0.4064339f, 0.1952015f) * colliderScale,
                new Vector2(0.3799641f, 0.1951965f) * colliderScale
            };
            var foreground = new GameObject("foreground");
            foreground.transform.SetParent(gameObject.transform);
            foreground.SetSprite(sprites[2], false, "Foreground", 3);
            foreground.transform.localScale = new Vector3(1, 1, 1);
            Utility.SetPositionWithOffSet(foreground.transform, gameObject.transform);
            foregroundSpriteRenderer = foreground.GetComponent<SpriteRenderer>();

            var damagableMachineryExample = ModAPI.FindSpawnable("Industrial Piston").Prefab.GetComponent<DamagableMachineryBehaviour>().ShallowClone();
            damageBehaviour = gameObject.AddComponent<DamagableMachineryBehaviour>();
            damageBehaviour.BreakDamage = 10;
            damageBehaviour.Waterproof = false;
            damageBehaviour.FirearmDamageMultiplier = 1.1f;
            damageBehaviour.ExplosionDamageMultiplier = 1.3f;
            damageBehaviour.BehavioursToToggle = new MonoBehaviour[] { };
            damageBehaviour.GameObjectsToToggle = new GameObject[] { };
            damageBehaviour.ExplosionPrefab = damagableMachineryExample.ExplosionPrefab;

            var actOnCollide = gameObject.AddComponent<ActOnCollideOwn>();
            actOnCollide.DispatchChance = 1f;
            actOnCollide.ImpactForceThreshold = 70f;
            actOnCollide.debug = false;
            actOnCollide.Actions = (collision) =>
            {
                if (!damageBehaviour.Destroyed)
                {
                    damageBehaviour.Slice();
                }
            };

            var door = ModAPI.CreatePhysicalObject("door", sprites[3]);
            door.transform.SetParent(gameObject.transform);
            Utility.SetPositionWithOffSet(door.transform, gameObject.transform, 0f, 0.6f);
            var hingeJoint = gameObject.AddComponent<HingeJoint2D>();
            hingeJoint.autoConfigureConnectedAnchor = false;
            hingeJoint.connectedBody = door.GetComponent<Rigidbody2D>();
            hingeJoint.anchor = new Vector2(-1.1f, 0.6f);
            hingeJoint.connectedAnchor = new Vector2(-1.1f, 0);
            hingeJoint.limits = new JointAngleLimits2D()
            {
                min = gameObject.transform.localScale.x > 0 ? 2 : -2,
                max = gameObject.transform.localScale.x > 0 ? 130 : -130
            };
            hingeJoint.useLimits = true;
            defaultMaterial = foregroundSpriteRenderer.material;
            brokenMaterial = Resources.Load<Material>("Materials/BrokenMachine");

            gameObject.GetComponent<Rigidbody2D>().mass = 5;

            internalCollider = Utility.CreateChildObject("internalCollider", gameObject.transform).AddComponent<BoxCollider2D>();
            Utility.SetPositionWithOffSet(internalCollider.transform, gameObject.transform);
            internalCollider.size = new Vector2(2.1f, 1f);
            door.transform.localScale = new Vector3(1f, 1f, 1f);
            gameObject.UpdateOutline();
        }
        private void Use()
        {
            foregroundTransparent = !foregroundTransparent;
            if (foregroundTransparent)
            {
                foregroundSpriteRenderer.sprite = sprites[1];
            }
            if (!foregroundTransparent)
            {
                foregroundSpriteRenderer.sprite = sprites[2];
            }
        }
        private void Update()
        {
            if (damageBehaviour.Destroyed)
            {
                foregroundSpriteRenderer.material = brokenMaterial;
            }
            if (!damageBehaviour.Destroyed)
            {
                foregroundSpriteRenderer.material = defaultMaterial;
                foreach (var collision in Utility.InCollider(internalCollider))
                {
                    if (collision.gameObject != gameObject)
                    {
                        if (collision.gameObject.TryGetComponent<PhysicalBehaviour>(out PhysicalBehaviour physicalBehaviour))
                        {
                            if (physicalBehaviour.Temperature > maxTemperature)
                            {
                                physicalBehaviour.Temperature = physicalBehaviour.Temperature - speed;
                            }
                        }
                    }
                }
            }
        }
    }
}
