﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TMPro;
using UnityEngine;

namespace Torture
{
    public class Microwave : MonoBehaviour
    {
        public Sprite[] sprites = new Sprite[] { };
        public AudioClip[] audioClips = new AudioClip[] { };
        public bool activate = false;
        public LightSprite light;
        public DamagableMachineryBehaviour damageBehaviour;
        public Collider2D internalCollider;
        public float maxTemperature = 300f;
        public float speed = 40f;
        public AudioSource rumbleSource;
        public AudioSource dingSource;
        public MicrowaveTimer microwaveTimer;
        private void Start()
        {
            gameObject.transform.localScale = Utility.GetModuleVector(gameObject.transform.localScale);
            Utility.SetSprite(gameObject, sprites[2]);
            gameObject.UpdateOutline();
            var background = Utility.CreateChildObject("background", gameObject.transform);
            background.SetSprite(sprites[0], false, "Background", -10);
            Utility.SetPositionWithOffSet(background.transform, gameObject.transform, -0.4f);
            var glass = Utility.CreateChildObject("glass", gameObject.transform);
            glass.SetSprite(sprites[1], false, "Foreground", 3);
            Utility.SetPositionWithOffSet(glass.transform, gameObject.transform);
            var physicalBehaviour = gameObject.GetComponent<PhysicalBehaviour>();
            physicalBehaviour.RefreshOutline();
            physicalBehaviour.rigidbody.mass = 1;
            var damagableMachineryExample = ModAPI.FindSpawnable("Industrial Piston").Prefab.GetComponent<DamagableMachineryBehaviour>().ShallowClone();
            damageBehaviour = gameObject.AddComponent<DamagableMachineryBehaviour>();
            damageBehaviour.BreakDamage = 10;
            damageBehaviour.Waterproof = false;
            damageBehaviour.FirearmDamageMultiplier = 1.1f;
            damageBehaviour.ExplosionDamageMultiplier = 1.3f;
            damageBehaviour.BehavioursToToggle = new MonoBehaviour[] { };
            damageBehaviour.GameObjectsToToggle = new GameObject[] { };
            damageBehaviour.ExplosionPrefab = damagableMachineryExample.ExplosionPrefab;

            var actOnCollide = gameObject.AddComponent<ActOnCollideOwn>();
            actOnCollide.DispatchChance = 1f;
            actOnCollide.ImpactForceThreshold = 10f;
            actOnCollide.debug = true;
            actOnCollide.Actions = (collision) =>
            {
                if (!damageBehaviour.Destroyed)
                {
                    ActStop();
                    microwaveTimer.StopTimer();
                }
            };

            light = ModAPI.CreateLight(gameObject.transform, Color.yellow, 10, 5);
            light.SpriteRenderer.enabled = false;

            var colliderHolder = Utility.CreateChildObject("holderCollider", gameObject.transform);
            var collider = colliderHolder.AddComponent<BoxCollider2D>();
            Utility.SetPositionWithOffSet(colliderHolder.transform, gameObject.transform, -0.38f);
            collider.size = new Vector2(2.85f, 1.76f);
            internalCollider = collider;

            rumbleSource = background.AddComponent<AudioSource>();
            rumbleSource.clip = audioClips[0];
            rumbleSource.loop = true;

            dingSource = background.AddComponent<AudioSource>();
            dingSource.clip = audioClips[1];
            background.AddComponent<AudioSourceTimeScaleBehaviour>();

            var timerObject = Utility.CreateChildObject("timer", gameObject.transform);
            Utility.SetPositionWithOffSet(timerObject.transform, gameObject.transform, 11.3636f, -1.6182f);
            microwaveTimer = timerObject.AddComponent<MicrowaveTimer>();
            microwaveTimer.damageBehaviour = damageBehaviour;
            microwaveTimer.microwave = this;
        }
        public void ActStop()
        {
            damageBehaviour.Slice();
            light.SpriteRenderer.enabled = false;
            dingSource.Play();
            rumbleSource.Stop();
            activate = false;
            microwaveTimer.StopTimer();
        }
        private void Use()
        {
            if (!damageBehaviour.Destroyed)
            {
                if (!activate && !damageBehaviour.Destroyed)
                {
                    light.SpriteRenderer.enabled = true;
                    rumbleSource.Play();
                    microwaveTimer.StartTimer();
                }
                else
                {
                    rumbleSource.Stop();
                    dingSource.Play();
                    light.SpriteRenderer.enabled = false;
                    microwaveTimer.StopTimer();
                }
                activate = !activate;
            }
        }
        private void Update()
        {
            if(activate && !damageBehaviour.Destroyed)
            {
                foreach(var collision in Utility.InCollider(internalCollider))
                {
                    if(collision.gameObject != gameObject)
                    {
                        if(collision.gameObject.TryGetComponent<PhysicalBehaviour>(out PhysicalBehaviour physicalBehaviour))
                        {
                            if(physicalBehaviour.Temperature < maxTemperature)
                            {
                                physicalBehaviour.Temperature = physicalBehaviour.Temperature + speed * Time.deltaTime;
                            }
                            if(physicalBehaviour.Properties.PhysicMaterial.name == "Metal")
                            {
                                if(UnityEngine.Random.Range(0, 100) > 98)
                                {
                                    ActStop();
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    public class MicrowaveTimer : MonoBehaviour
    {
        public TextMeshPro textMeshPro;
        public float totalSeconds;
        public bool active = false;
        public DamagableMachineryBehaviour damageBehaviour;
        public Microwave microwave;
        private void Start()
        {
            textMeshPro = gameObject.AddComponent<TextMeshPro>();
            textMeshPro.font = Utility.GetTMP_FontAsset("Clock");
            textMeshPro.color = Color.red;
            textMeshPro.renderer.sortingLayerName = "Foreground";
            textMeshPro.fontSize = 2;
            textMeshPro.text = "00:00";
        }
        public void StartTimer()
        {
            totalSeconds = 0;
            active = true;
        }
        public void StopTimer()
        {
            active = false;
            textMeshPro.text = "00:00";
        }
        private void Update()
        {
            if (active && !damageBehaviour.Destroyed)
            {
                totalSeconds += Time.deltaTime;
                var countdownTime = TimeSpan.FromSeconds(totalSeconds);
                textMeshPro.text = string.Format("{0:00}:{1:00}", countdownTime.Minutes, countdownTime.Seconds);
                if(textMeshPro.text == "59:99")
                {
                    microwave.ActStop();
                }
            }
            if (damageBehaviour.Destroyed)
            {
                textMeshPro.enabled = false;
            }
            if (!damageBehaviour.Destroyed)
            {
                textMeshPro.enabled = true;
            }
        }
    }
}
