﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;

namespace Torture
{
    public class Shocker : MonoBehaviour
    {
        public Sprite[] sprites = new Sprite[] { };
        public BoxCollider2D boxCollider;
        public GameObject particle;
        public ParticleSystem[] particleSystems;
        public PhysicalBehaviour physicalBehaviour;
        public bool activate = false;
        public float charge = 5f;
        public AudioClip[] audioClips = new AudioClip[] { };
        public AudioSource audioSource;

        private void Start()
        {
            gameObject.SetSprite(sprites[0], false);
            UnityEngine.Object.Destroy(gameObject.GetComponent<Collider2D>());
            var colliderShocker = gameObject.AddComponent<BoxCollider2D>();
            colliderShocker.size = new Vector2(0.08f, 1f);
            colliderShocker.offset = new Vector2(0.012f, -0.1f);
            var colliderHandler = Utility.CreateChildObject("colliderHandler", gameObject.transform);
            Utility.SetPositionWithOffSet(colliderHandler.transform, gameObject.transform);
            boxCollider = colliderHandler.AddComponent<BoxCollider2D>();
            boxCollider.isTrigger = true;
            boxCollider.size = new Vector2(0.15f, 0.2f);
            boxCollider.offset = new Vector2(0.012f, 0.4f);
            var particlePrefab = ModAPI.FindSpawnable("Generator").Prefab.transform.Find("P1").gameObject;
            particle = Instantiate(particlePrefab, gameObject.transform);
            particle.transform.localScale = new Vector3(0.1f, -0.1f, 1);
            Utility.SetPositionWithOffSet(particle.transform, gameObject.transform, 0.013f, 0.43f);
            physicalBehaviour = gameObject.GetComponent<PhysicalBehaviour>();
            particleSystems = new ParticleSystem[] { particle.GetComponent<ParticleSystem>(), particle.transform.Find("SpriteLightFlash").GetComponent<ParticleSystem>(), particle.transform.Find("P2").GetComponent<ParticleSystem>() };

            audioSource = particle.AddComponent<AudioSource>();
            audioSource.loop = true;
            audioSource.clip = audioClips[0];
            particle.AddComponent<AudioSourceTimeScaleBehaviour>();
            ToggleShocker(false);
            gameObject.UpdateOutline();
            physicalBehaviour.ResetColliderArray();
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            if (Utility.InCollider(boxCollider).Contains(collision.collider) && activate)
            {
                if(collision.gameObject.TryGetComponent<LimbBehaviour>(out LimbBehaviour limbBehaviour))
                {
                    limbBehaviour.SkinMaterialHandler.AddDamagePoint(DamageType.Burn, collision.contacts[0].point, 10f);
                }
            }
        }
        private void ToggleShocker(bool activate)
        {
            this.activate = activate;
            foreach(var particleSystem in particleSystems)
            {
                if (activate)
                {
                    audioSource.Play();
                    particleSystem.Play();
                    foreach(var obj in Utility.InCollider(boxCollider))
                    {
                        if(obj.gameObject != transform.root.gameObject)
                        {
                            if (obj.gameObject.TryGetComponent<PhysicalBehaviour>(out PhysicalBehaviour physicalBehaviour))
                            {
                                physicalBehaviour.charge = charge;
                            }
                        }
                    }
                }
                else
                {
                    audioSource.Stop();
                    particleSystem.Stop();
                }
            }
        }
        private void Update()
        {
            if (physicalBehaviour.IsBeingUsedContinuously())
            {
                ToggleShocker(true);
            }
            else
            {
                ToggleShocker(false);
            }
        }
    }
}
