﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;

namespace Torture
{
    public class WaterTortureCell : MonoBehaviour 
    {
        public Sprite[] sprites = new Sprite[] { };
        public Transform seaObject;
        public LiquidWater liquidWater;
        public string lastLiquidDisplayName = "Acid";
        public string lastType = "water";
        public void Start()
        {
            gameObject.SetSprite(sprites[0], true, "Top", 0);
            UnityEngine.Object.Destroy(gameObject.GetComponent<Collider2D>());
            var collider = gameObject.AddComponent<PolygonCollider2D>();
            float colliderSize = 2.85f;
            collider.SetPath(0, new Vector2[]
            {
                new Vector2(-0.2443686f, 0.4996116f) * colliderSize,
                new Vector2(-0.2441721f, -0.4994173f) * colliderSize,
                new Vector2(0.2436447f, -0.4994392f) * colliderSize,
                new Vector2(0.2431536f, 0.4998443f) * colliderSize
            });
            collider.SetPath(1, new Vector2[] 
            {
                new Vector2(-0.2052485f, 0.4252485f) * colliderSize,
                new Vector2(0.2053744f, 0.4254483f) * colliderSize,
                new Vector2(0.206001f, -0.4646439f) * colliderSize,
                new Vector2(-0.2055051f, -0.4651069f) * colliderSize
            });
            var seaPrefab = MapRegistry.GetMap("35a41d94-8bc7-47a2-93de-3a297955e97c").Prefab.transform.Find("Root/Sea");
            seaObject = Instantiate(seaPrefab, gameObject.transform);
            Utility.SetPositionWithOffSet(seaObject.transform, gameObject.transform, 0, -0.02f);
            seaObject.localScale = new Vector3(1.2761f, - 2.7392f, 1);
            seaObject.GetComponent<WaterBehaviour>().ObjectsToBeActiveIfWater[1] = null;
            seaObject.transform.Find("Ambience").gameObject.SetActive(false);
            gameObject.GetComponent<Rigidbody2D>().mass = 30f;

            liquidWater = seaObject.gameObject.AddComponent<LiquidWater>();
            gameObject.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton(() => !seaObject.gameObject.activeInHierarchy || liquidWater.activate, "fillWater", "Fill Water", "", () => SetLiquid("water")));
            gameObject.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton(() => !liquidWater.activate, "fillLiquid", () => "Fill " + Liquid.GetLiquid(liquidWater.nowLiquidID).GetDisplayName(), "", () => SetLiquid("liquid")));
            gameObject.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton(() => !liquidWater.activate, "changeLiquid", "Change Liquid", "", () => 
            {
                DialogBox dialogBox = (DialogBox)null;
                dialogBox = DialogBoxManager.TextEntry("Change liquid", "<size=75%>Enter the name of the liquid\nyou can get it in the liquid analyzer", new DialogButton[]
                {
                    new DialogButton("Cancel", true, () => dialogBox.Close()),
                    new DialogButton("Enter", true, () => liquidWater.ChangeLiquid(dialogBox.EnteredText))
                });
            }));
            liquidWater.waterTorture = this;
            gameObject.UpdateOutline();
        }
        public void SetLiquid(string liquid)
        {
            switch (liquid)
            {
                case "water":
                    {
                        liquidWater.Fill(false);
                        seaObject.gameObject.SetActive(true);
                        break;
                    }
                case "liquid":
                    {
                        liquidWater.Fill(true);
                        seaObject.gameObject.SetActive(true);
                        break;
                    }
            }
            lastType = liquid;
        }
    }
    public class LiquidWater : MonoBehaviour
    {
        public bool activate = false;
        public Collider2D collider;
        public Color normalColor = new Color(0.165f, 0.3197f, 0.3396f, 0.6078f);
        public Color nowColor = new Color(0, 0.6703f, 0, 0.7878f);
        public Material material;
        public string nowLiquidID = "ACID";
        public Dictionary<string, Liquid> allLiquidsOfName = new Dictionary<string, Liquid>();
        public WaterTortureCell waterTorture;
        private void Start()
        {
            collider = gameObject.GetComponent<Collider2D>();
            material = gameObject.GetComponent<SpriteRenderer>().material;
            foreach (var liquid in Liquid.GetLiquid(nowLiquidID).GetAll())
            {
                try
                {
                    if (!allLiquidsOfName.ContainsKey(liquid.GetDisplayName().ToLower()))
                    {
                        allLiquidsOfName.Add(liquid.GetDisplayName().ToLower(), liquid);
                    }
                }
                catch { }
            }
            ChangeLiquid(waterTorture.lastLiquidDisplayName);
            waterTorture.SetLiquid(waterTorture.lastType);
        }
        private void Update()
        {
            
            if (activate)
            {
                foreach(var obj in Utility.InCollider(collider))
                {
                    if(obj.gameObject.TryGetComponent<LimbBehaviour>(out LimbBehaviour limbBehaviour))
                    {
                        limbBehaviour.CirculationBehaviour.AddLiquid(Liquid.GetLiquid(nowLiquidID), 0.05f * Time.deltaTime);
                    }
                }
            }
        }
        public void Fill(bool fill = true)
        {
            activate = fill;
            nowColor = new Color(nowColor.r, nowColor.g, nowColor.b, 0.6f);
            material.SetColor("_Tint", activate == true ? nowColor : normalColor);
            material.SetColor("_DepthColour", activate == true ? nowColor : normalColor);
        }
        public void ChangeLiquid(string displayLiquidName, bool notify = true)
        {
            if (allLiquidsOfName.ContainsKey(displayLiquidName.ToLower()))
            {
                var liquid = allLiquidsOfName[displayLiquidName.ToLower()];
                nowLiquidID = Liquid.GetIdentity(liquid);
                nowColor = liquid.Color;
                if (notify)
                {
                    Utility.SuccessfulNotify("You changed the liquid to " + displayLiquidName);
                }
                waterTorture.lastLiquidDisplayName = displayLiquidName;
            }
            else
            {
                if (notify)
                {
                    Utility.ErrorNotify("Incorrect name of the liquid");
                }
            }
        }
    }
}
